package me.totalfreedom.provider;

import net.kyori.adventure.text.Component;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Objects;
import java.util.stream.Stream;

public class ContextProvider
{
    public Object fromString(final String string)
    {
        return Stream.of(toBoolean(string),
                        toDouble(string),
                        toInt(string),
                        toLong(string),
                        toFloat(string),
                        toPlayer(string),
                        toWorld(string),
                        toLocation(string),
                        toCommandSender(string),
                        toComponent(string))
                .filter(Objects::nonNull)
                .findFirst()
                .orElse(string);
    }

    private @Nullable Boolean toBoolean(final String string)
    {
        try
        {
            return Boolean.parseBoolean(string);
        } catch (Exception e)
        {
            return null;
        }
    }

    private @Nullable Double toDouble(final String string)
    {
        try
        {
            return Double.parseDouble(string);
        } catch (Exception e)
        {
            return null;
        }
    }

    private @Nullable Integer toInt(final String string)
    {
        try
        {
            return Integer.parseInt(string);
        } catch (Exception e)
        {
            return null;
        }
    }

    private @Nullable Long toLong(final String string)
    {
        try
        {
            return Long.parseLong(string);
        } catch (Exception e)
        {
            return null;
        }
    }

    private @Nullable Float toFloat(final String string)
    {
        try
        {
            return Float.parseFloat(string);
        } catch (Exception e)
        {
            return null;
        }
    }

    private @Nullable Player toPlayer(final String string)
    {
        return Bukkit.getPlayer(string);
    }

    private @Nullable CommandSender toCommandSender(final String string)
    {
        if (toPlayer(string) == null) return null;

        return toPlayer(string);
    }

    private @Nullable World toWorld(final String string)
    {
        return Bukkit.getWorld(string);
    }

    // If we decide to, we can "modify" this to use spaces
    // and adjust our inputs accordingly.
    /**
     * When using this method, the input string must be formatted as
     * <br>
     * <code>worldName,x,y,z</code>
     * <br>
     *
     * @param string The string to parse
     * @return A location object if xyz is valid
     */
    private @Nullable Location toLocation(final String string)
    {
        final String[] split = string.split(",");

        if (split.length != 4 || toWorld(split[0]) == null) return null;

        final double x = Double.parseDouble(split[1]);
        final double y = Double.parseDouble(split[2]);
        final double z = Double.parseDouble(split[3]);

        return new Location(toWorld(split[0]), x, y, z);
    }

    private @NotNull Component toComponent(final String string)
    {
        return Component.text(string);
    }
}
